'''
The :mod:`kitchen.pycompat24` module contains implementations of functionality
introduced in python-2.4 for use on earlier versions of python.
'''
import warnings

warnings.warn('In python3, kitchen.pycompat24 is deprecated because the'
        ' python stdlib has this code in all python3 versions.  If your code'
        ' doesn\'t have to remain compatible with python less than 2.4 use'
        ' python3\'s stdlib versions of base64, subprocess, and the builtin'
        ' set types instead',
        PendingDeprecationWarning, stacklevel=2)

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 1, 0),)
__version__ = version_tuple_to_string(__version_info__)

__all__ = ('base64', 'sets', 'subprocess')



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools
from nose.plugins.skip import SkipTest

try:
    import chardet
except ImportError:
    chardet = None

from kitchen.text import misc
from kitchen.text.exceptions import ControlCharError
from kitchen.text.converters import to_unicode

import base_classes

class TestTextMisc(unittest.TestCase, base_classes.UnicodeTestData):
    def test_guess_encoding_no_chardet(self):
        # Test that unicode strings are not allowed
        tools.assert_raises(TypeError, misc.guess_encoding, self.u_spanish)

        tools.ok_(misc.guess_encoding(self.utf8_spanish, disable_chardet=True) == 'utf-8')
        tools.ok_(misc.guess_encoding(self.latin1_spanish, disable_chardet=True) == 'latin-1')
        tools.ok_(misc.guess_encoding(self.utf8_japanese, disable_chardet=True) == 'utf-8')
        tools.ok_(misc.guess_encoding(self.euc_jp_japanese, disable_chardet=True) == 'latin-1')

    def test_guess_encoding_with_chardet(self):
        # We go this slightly roundabout way because multiple encodings can
        # output the same byte sequence.  What we're really interested in is
        # if we can get the original unicode string without knowing the
        # converters beforehand
        tools.ok_(to_unicode(self.utf8_spanish,
            misc.guess_encoding(self.utf8_spanish)) == self.u_spanish)
        tools.ok_(to_unicode(self.latin1_spanish,
            misc.guess_encoding(self.latin1_spanish)) == self.u_spanish)
        tools.ok_(to_unicode(self.utf8_japanese,
            misc.guess_encoding(self.utf8_japanese)) == self.u_japanese)

    def test_guess_encoding_with_chardet_installed(self):
        if chardet:
            tools.ok_(to_unicode(self.euc_jp_japanese,
                misc.guess_encoding(self.euc_jp_japanese)) == self.u_japanese)
        else:
            raise SkipTest('chardet not installed, euc_jp will not be guessed correctly')

    def test_guess_encoding_with_chardet_uninstalled(self):
        if chardet:
            raise SkipTest('chardet installed, euc_jp will not be mangled')
        else:
            tools.ok_(to_unicode(self.euc_jp_japanese,
                misc.guess_encoding(self.euc_jp_japanese)) ==
                self.u_mangled_euc_jp_as_latin1)

    def test_str_eq(self):
        # str vs str:
        tools.ok_(misc.str_eq(self.euc_jp_japanese, self.euc_jp_japanese) == True)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.utf8_japanese) == True)
        tools.ok_(misc.str_eq(self.b_ascii, self.b_ascii) == True)
        tools.ok_(misc.str_eq(self.euc_jp_japanese, self.latin1_spanish) == False)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.euc_jp_japanese) == False)
        tools.ok_(misc.str_eq(self.b_ascii, self.b_ascii[:-2]) == False)

        # unicode vs unicode:
        tools.ok_(misc.str_eq(self.u_japanese, self.u_japanese) == True)
        tools.ok_(misc.str_eq(self.u_ascii, self.u_ascii) == True)
        tools.ok_(misc.str_eq(self.u_japanese, self.u_spanish) == False)
        tools.ok_(misc.str_eq(self.u_ascii, self.u_ascii[:-2]) == False)

        # unicode vs str with default utf-8 conversion:
        tools.ok_(misc.str_eq(self.u_japanese, self.utf8_japanese) == True)
        tools.ok_(misc.str_eq(self.u_ascii, self.b_ascii) == True)
        tools.ok_(misc.str_eq(self.u_japanese, self.euc_jp_japanese) == False)
        tools.ok_(misc.str_eq(self.u_ascii, self.b_ascii[:-2]) == False)

        # unicode vs str with explicit encodings:
        tools.ok_(misc.str_eq(self.u_japanese, self.euc_jp_japanese, encoding='euc_jp') == True)
        tools.ok_(misc.str_eq(self.u_japanese, self.utf8_japanese, encoding='utf8') == True)
        tools.ok_(misc.str_eq(self.u_ascii, self.b_ascii, encoding='latin1') == True)
        tools.ok_(misc.str_eq(self.u_japanese, self.euc_jp_japanese, encoding='latin1') == False)
        tools.ok_(misc.str_eq(self.u_japanese, self.utf8_japanese, encoding='euc_jp') == False)
        tools.ok_(misc.str_eq(self.u_japanese, self.utf8_japanese, encoding='euc_jp') == False)
        tools.ok_(misc.str_eq(self.u_ascii, self.b_ascii[:-2], encoding='latin1') == False)

        # str vs unicode (reverse parameter order of unicode vs str)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.u_japanese) == True)
        tools.ok_(misc.str_eq(self.b_ascii, self.u_ascii) == True)
        tools.ok_(misc.str_eq(self.euc_jp_japanese, self.u_japanese) == False)
        tools.ok_(misc.str_eq(self.b_ascii, self.u_ascii[:-2]) == False)

        tools.ok_(misc.str_eq(self.euc_jp_japanese, self.u_japanese, encoding='euc_jp') == True)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.u_japanese, encoding='utf8') == True)
        tools.ok_(misc.str_eq(self.b_ascii, self.u_ascii, encoding='latin1') == True)
        tools.ok_(misc.str_eq(self.euc_jp_japanese, self.u_japanese, encoding='latin1') == False)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.u_japanese, encoding='euc_jp') == False)
        tools.ok_(misc.str_eq(self.utf8_japanese, self.u_japanese, encoding='euc_jp') == False)
        tools.ok_(misc.str_eq(self.b_ascii, self.u_ascii[:-2], encoding='latin1') == False)


    def test_process_control_chars(self):
        tools.assert_raises(TypeError, misc.process_control_chars, b'byte string')
        tools.assert_raises(ControlCharError, misc.process_control_chars,
                *[self.u_ascii_chars], **{'strategy': 'strict'})
        tools.ok_(misc.process_control_chars(self.u_ascii_chars,
            strategy='ignore') == self.u_ascii_no_ctrl)
        tools.ok_(misc.process_control_chars(self.u_ascii_chars,
            strategy='replace') == self.u_ascii_ctrl_replace)

    def test_html_entities_unescape(self):
        tools.assert_raises(TypeError, misc.html_entities_unescape, b'byte string')
        tools.ok_(misc.html_entities_unescape(self.u_entity_escape) == self.u_entity)
        tools.ok_(misc.html_entities_unescape('<tag>%s</tag>'
            % self.u_entity_escape) == self.u_entity)
        tools.ok_(misc.html_entities_unescape('a&#1234567890;b') == 'a&#1234567890;b')
        tools.ok_(misc.html_entities_unescape('a&#xfffd;b') == 'a\ufffdb')
        tools.ok_(misc.html_entities_unescape('a&#65533;b') == 'a\ufffdb')

    def test_byte_string_valid_xml(self):
        tools.ok_(misc.byte_string_valid_xml('unicode string') == False)

        tools.ok_(misc.byte_string_valid_xml(self.utf8_japanese))
        tools.ok_(misc.byte_string_valid_xml(self.euc_jp_japanese, 'euc_jp'))

        tools.ok_(misc.byte_string_valid_xml(self.utf8_japanese, 'euc_jp') == False)
        tools.ok_(misc.byte_string_valid_xml(self.euc_jp_japanese, 'utf8') == False)

        tools.ok_(misc.byte_string_valid_xml(self.utf8_ascii_chars) == False)

    def test_byte_string_valid_encoding(self):
        '''Test that a byte sequence is validated'''
        tools.ok_(misc.byte_string_valid_encoding(self.utf8_japanese) == True)
        tools.ok_(misc.byte_string_valid_encoding(self.euc_jp_japanese, encoding='euc_jp') == True)

    def test_byte_string_invalid_encoding(self):
        '''Test that we return False with non-encoded chars'''
        tools.ok_(misc.byte_string_valid_encoding(b'\xff') == False)
        tools.ok_(misc.byte_string_valid_encoding(self.euc_jp_japanese) == False)

class TestIsStringTypes(unittest.TestCase):
    def test_isbasestring(self):
        tools.assert_true(misc.isbasestring(b'abc'))
        tools.assert_true(misc.isbasestring('abc'))
        tools.assert_false(misc.isbasestring(5))

    def test_isbytestring(self):
        tools.assert_true(misc.isbytestring(b'abc'))
        tools.assert_false(misc.isbytestring('abc'))
        tools.assert_false(misc.isbytestring(5))

    def test_isunicodestring(self):
        tools.assert_false(misc.isunicodestring(b'abc'))
        tools.assert_true(misc.isunicodestring('abc'))
        tools.assert_false(misc.isunicodestring(5))



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools

from kitchen import iterutils

class TestIterutils(unittest.TestCase):
    iterable_data = (
            [0, 1, 2],
            [],
            (0, 1, 2),
            tuple(),
            set([0, 1, 2]),
            set(),
            dict(a=1, b=2),
            dict(),
            [None],
            [False],
            [0],
            xrange(0, 3),
            iter([1, 2, 3]),
            )
    non_iterable_data = (
            None,
            False,
            True,
            0,
            1.1,
            )

    def test_isiterable(self):
        for item in self.iterable_data:
            tools.ok_(iterutils.isiterable(item) == True)

        for item in self.non_iterable_data:
            tools.ok_(iterutils.isiterable(item) == False)

        # strings
        tools.ok_(iterutils.isiterable('a', include_string=True) == True)
        tools.ok_(iterutils.isiterable('a', include_string=False) == False)
        tools.ok_(iterutils.isiterable('a') == False)
        tools.ok_(iterutils.isiterable(u'a', include_string=True) == True)
        tools.ok_(iterutils.isiterable(u'a', include_string=False) == False)
        tools.ok_(iterutils.isiterable(u'a') == False)

    def test_iterate(self):
        iterutils.iterate(None)
        for item in self.non_iterable_data:
            tools.ok_(list(iterutils.iterate(item)) == [item])

        for item in self.iterable_data[:-1]:
            tools.ok_(list(iterutils.iterate(item)) == list(item))

        # iter() is exhausted after use so we have to test separately
        tools.ok_(list(iterutils.iterate(iter([1, 2, 3]))) == [1, 2, 3])

        # strings
        tools.ok_(list(iterutils.iterate('abc')) == ['abc'])
        tools.ok_(list(iterutils.iterate('abc', include_string=True)) == ['a', 'b', 'c'])
        tools.ok_(list(iterutils.iterate(u'abc')) == [u'abc'])
        tools.ok_(list(iterutils.iterate(u'abc', include_string=True)) == [u'a', u'b', u'c'])



'''
The :mod:`kitchen.pycompat25` module contains implementations of functionality
introduced in python-2.5.
'''

from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 0, 0),)
__version__ = version_tuple_to_string(__version_info__)


__all__ = ('collections',)



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools

import warnings

from kitchen.text import utf8

import base_classes

class TestUTF8(base_classes.UnicodeTestData, unittest.TestCase):
    def setUp(self):
        # All of the utf8* functions are deprecated
        warnings.simplefilter('ignore', DeprecationWarning)

    def tearDown(self):
        warnings.simplefilter('default', DeprecationWarning)

    def test_utf8_width(self):
        '''Test that we find the proper number of spaces that a utf8 string will consume'''
        tools.ok_(utf8.utf8_width(self.utf8_japanese) == 31)
        tools.ok_(utf8.utf8_width(self.utf8_spanish) == 50)
        tools.ok_(utf8.utf8_width(self.utf8_mixed) == 23)

    def test_utf8_width_non_utf8(self):
        '''Test that we handle non-utf8 bytes in utf8_width without backtracing'''
        # utf8_width() treats non-utf8 byte sequences as undecodable so you
        # end up with less characters than normal.  In this string:
        # Python-2.7+ replaces problematic characters in a different manner
        # than older pythons.
        # Python >= 2.7:
        # El veloz murci�lago salt� sobre el perro perezoso.
        # Python < 2.7:
        # El veloz murci�go salt�bre el perro perezoso.
        if len(str('\xe9la'.encode('latin1'), 'utf8', 'replace')) == 1:
            # Python < 2.7
            tools.ok_(utf8.utf8_width(self.latin1_spanish) == 45)
        else:
            # Python >= 2.7
            tools.ok_(utf8.utf8_width(self.latin1_spanish) == 50)

    def test_utf8_width_chop(self):
        '''utf8_width_chop with byte strings'''
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed) == (23, self.utf8_mixed))
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed, 23) == (23, self.utf8_mixed))
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed, 22) == (22, self.utf8_mixed[:-1]))
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed, 19) == (18, self.u_mixed[:-4].encode('utf8')))
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed, 2) == (2, self.u_mixed[0].encode('utf8')))
        tools.ok_(utf8.utf8_width_chop(self.utf8_mixed, 1) == (0, b''))

    def test_utf8_width_chop_unicode(self):
        '''utf8_width_chop with unicode input'''
        tools.ok_(utf8.utf8_width_chop(self.u_mixed) == (23, self.u_mixed))
        tools.ok_(utf8.utf8_width_chop(self.u_mixed, 23) == (23, self.u_mixed))
        tools.ok_(utf8.utf8_width_chop(self.u_mixed, 22) == (22, self.u_mixed[:-1]))
        tools.ok_(utf8.utf8_width_chop(self.u_mixed, 19) == (18, self.u_mixed[:-4]))
        tools.ok_(utf8.utf8_width_chop(self.u_mixed, 2) == (2, self.u_mixed[0]))
        tools.ok_(utf8.utf8_width_chop(self.u_mixed, 1), (0, ''))

    def test_utf8_width_fill(self):
        '''Pad a utf8 string'''
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 1) == self.utf8_mixed)
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 25) == self.utf8_mixed + b'  ')
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 25, left=False) == b'  ' + self.utf8_mixed)
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 25, chop=18) == self.u_mixed[:-4].encode('utf8') + b'       ')
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 25, chop=18, prefix=self.utf8_spanish, suffix=self.utf8_spanish) == self.utf8_spanish + self.u_mixed[:-4].encode('utf8') + self.utf8_spanish + b'       ')
        tools.ok_(utf8.utf8_width_fill(self.utf8_mixed, 25, chop=18) == self.u_mixed[:-4].encode('utf8') + b'       ')
        tools.ok_(utf8.utf8_width_fill(self.u_mixed, 25, chop=18, prefix=self.u_spanish, suffix=self.utf8_spanish) == self.u_spanish.encode('utf8') + self.u_mixed[:-4].encode('utf8') + self.u_spanish.encode('utf8') + b'       ')
        pass

    def test_utf8_valid(self):
        '''Test that a utf8 byte sequence is validated'''
        warnings.simplefilter('ignore', DeprecationWarning)
        tools.ok_(utf8.utf8_valid(self.utf8_japanese) == True)
        tools.ok_(utf8.utf8_valid(self.utf8_spanish) == True)
        warnings.simplefilter('default', DeprecationWarning)

    def test_utf8_invalid(self):
        '''Test that we return False with non-utf8 chars'''
        warnings.simplefilter('ignore', DeprecationWarning)
        tools.ok_(utf8.utf8_valid(b'\xff') == False)
        tools.ok_(utf8.utf8_valid(self.latin1_spanish) == False)
        warnings.simplefilter('default', DeprecationWarning)

    def test_utf8_text_wrap(self):
        tools.ok_(utf8.utf8_text_wrap(self.utf8_mixed) == [self.utf8_mixed])
        tools.ok_(utf8.utf8_text_wrap(self.utf8_paragraph) == self.utf8_paragraph_out)
        tools.ok_(utf8.utf8_text_wrap(self.utf8_mixed_para) == self.utf8_mixed_para_out)
        tools.ok_(utf8.utf8_text_wrap(self.utf8_mixed_para, width=57,
            initial_indent=b'    ', subsequent_indent=b'----') ==
            self.utf8_mixed_para_57_initial_subsequent_out)



#!/usr/bin/python -tt

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import glob
import os
import shutil

try:
    # Only availabe on py2
    from kitchen.pycompat27 import subprocess
except:
    import subprocess


class MsgFmt(object):
    def run(self, args):
        cmd = subprocess.Popen(args, shell=False)
        cmd.wait()


def setup_message_compiler():
    # Look for msgfmt
    try:
        subprocess.Popen(['msgfmt', '-h'], stdout=subprocess.PIPE)
    except OSError:
        import babel.messages.frontend

        return (babel.messages.frontend.CommandLineInterface(),
            'pybabel compile -D %(domain)s -d locale -i %(pofile)s -l %(lang)s'
            )
    else:
        return (MsgFmt(), 'msgfmt -c -o locale/%(lang)s/LC_MESSAGES/%(domain)s.mo %(pofile)s')


def main():
    # Get the directory with message catalogs
    # Reuse transifex's config file first as it will know this
    cfg = configparser.SafeConfigParser()
    cfg.read('.tx/config')
    cmd, args = setup_message_compiler()

    try:
        shutil.rmtree('locale')
    except OSError as e:
        # If the error is that locale does not exist, we're okay.  We're
        # deleting it here, afterall
        if e.errno != 2:
            raise

    for section in [s for s in cfg.sections() if s != 'main']:
        try:
            file_filter = cfg.get(section, 'file_filter')
            source_file = cfg.get(section, 'source_file')
        except configparser.NoOptionError:
            continue
        glob_pattern = file_filter.replace('<lang>', '*')
        pot = os.path.basename(source_file)
        if pot.endswith('.pot'):
            pot = pot[:-4]
        arg_values = {'domain': pot}
        for po_file in glob.glob(glob_pattern):
            file_pattern = os.path.basename(po_file)
            lang = file_pattern.replace('.po', '')
            os.makedirs(os.path.join('locale', lang, 'LC_MESSAGES'))
            arg_values['pofile'] = po_file
            arg_values['lang'] = lang
            compile_args = args % arg_values
            compile_args = compile_args.split(' ')
            cmd.run(compile_args)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from nose import tools

import sys
import warnings

import importlib
from kitchen.pycompat25.collections import defaultdict

class TestPendingDeprecationModules(object):
    def __init__(self):
        kitchen_path = 'kitchen'
        collections_path = 'kitchen/collections'
        pycompat24_path = 'kitchen/pycompat24'
        pycompat25_path = 'kitchen/pycompat25'
        pycompat27_path = 'kitchen/pycompat27'

        self.module_data = (
            ('strictdict', 'kitchen.collections.strictdict', collections_path),
            ('pycompat24', 'kitchen.pycompat24', kitchen_path),
            ('base64', 'kitchen.pycompat24.base64', pycompat24_path),
            ('sets', 'kitchen.pycompat24.sets', pycompat24_path),
            ('subprocess', 'kitchen.pycompat24.subprocess', pycompat24_path),
            ('pycompat25', 'kitchen.pycompat25', kitchen_path),
            ('collections', 'kitchen.pycompat25.collections', pycompat25_path),
            ('pycompat27', 'kitchen.pycompat27', kitchen_path),
            ('subprocess', 'kitchen.pycompat27.subprocess', pycompat27_path),
            )

    def setUp(self):
        for module in sys.modules.values():
            if hasattr(module, '__warningregistry__'):
                del module.__warningregistry__

    def check_modules(self, module_name, module_fqn, module_path):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')
            # imp.load_module will load even if it has already been loaded.
            # We need to ensure that happens in order to trigger the
            # deprecation warnings
            importlib.find_loader(module_fqn, module_path).load_module()
            warning_raised = False
            for warning in (e.message for e in w):
                if isinstance(warning, PendingDeprecationWarning) and \
                        ('%s is deprecated' % module_name) in warning.args[0]:
                    warning_raised = True
                    break
            tools.assert_true(warning_raised, msg='%s did not raise a PendingDeprecationWarning' % module_fqn)

    def test_modules(self):
        for mod in self.module_data:
            yield self.check_modules, mod[0], mod[1], mod[2]

    def test_defaultdict(self):
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter('always')
            defaultdict()
            warning_raised = False
            for warning in (e.message for e in w):
                if isinstance(warning, PendingDeprecationWarning) and \
                        ('defaultdict is deprecated') in warning.args[0]:
                    warning_raised = True
                    break
            tools.assert_true(warning_raised, msg='kitchen.pycompat25.collections.defaultdict did not raise a PendingDeprecationWarning')



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools

import os
import types

from kitchen import i18n

import base_classes

class TestI18N_UTF8(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_easy_gettext_setup(self):
        '''Test that the easy_gettext_setup function works
        '''
        _, N_ = i18n.easy_gettext_setup('foo', localedirs=
                ['%s/data/locale/' % os.path.dirname(__file__)])
        tools.assert_true(isinstance(_, types.MethodType))
        tools.assert_true(isinstance(N_, types.MethodType))
        tools.eq_(_.__name__, '_ugettext')
        tools.eq_(N_.__name__, '_ungettext')

        tools.eq_(_(self.utf8_spanish), self.u_spanish)
        tools.eq_(_(self.u_spanish), self.u_spanish)
        tools.eq_(N_(self.utf8_limao, self.utf8_limoes, 1), self.u_limao)
        tools.eq_(N_(self.utf8_limao, self.utf8_limoes, 2), self.u_limoes)
        tools.eq_(N_(self.u_limao, self.u_limoes, 1), self.u_limao)
        tools.eq_(N_(self.u_limao, self.u_limoes, 2), self.u_limoes)

    def test_easy_gettext_setup_non_unicode(self):
        '''Test that the easy_gettext_setup function works
        '''
        b_, bN_ = i18n.easy_gettext_setup('foo', localedirs=
                ['%s/data/locale/' % os.path.dirname(__file__)],
                use_unicode=False)
        tools.assert_true(isinstance(b_, types.MethodType))
        tools.assert_true(isinstance(bN_, types.MethodType))
        tools.eq_(b_.__name__, '_lgettext')
        tools.eq_(bN_.__name__, '_lngettext')

        tools.eq_(b_(self.utf8_spanish), self.utf8_spanish)
        tools.eq_(b_(self.u_spanish), self.utf8_spanish)
        tools.eq_(bN_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_limao)
        tools.eq_(bN_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_limoes)
        tools.eq_(bN_(self.u_limao, self.u_limoes, 1), self.utf8_limao)
        tools.eq_(bN_(self.u_limao, self.u_limoes, 2), self.utf8_limoes)

    def test_get_translation_object(self):
        '''Test that the get_translation_object function works
        '''
        translations = i18n.get_translation_object('foo', ['%s/data/locale/' % os.path.dirname(__file__)])
        tools.eq_(translations.__class__, i18n.DummyTranslations)
        tools.assert_raises(IOError, i18n.get_translation_object, 'foo', ['%s/data/locale/' % os.path.dirname(__file__)], fallback=False)

        translations = i18n.get_translation_object('test', ['%s/data/locale/' % os.path.dirname(__file__)])
        tools.eq_(translations.__class__, i18n.NewGNUTranslations)

    def test_get_translation_object_create_fallback(self):
        '''Test get_translation_object creates fallbacks for additional catalogs'''
        translations = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])
        tools.eq_(translations.__class__, i18n.NewGNUTranslations)
        tools.eq_(translations._fallback.__class__, i18n.NewGNUTranslations)

    def test_get_translation_object_copy(self):
        '''Test get_translation_object shallow copies the message catalog'''
        translations = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)], codeset='utf-8')
        translations.input_charset = 'utf-8'
        translations2 = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)], codeset='latin-1')
        translations2.input_charset = 'latin-1'

        # Test that portions of the translation objects are the same and other
        # portions are different (which is a space optimization so that the
        # translation data isn't in memory multiple times)
        tools.assert_not_equal(id(translations._fallback), id(translations2._fallback))
        tools.assert_not_equal(id(translations.output_charset()), id(translations2.output_charset()))
        tools.assert_not_equal(id(translations.input_charset), id(translations2.input_charset))
        tools.assert_not_equal(id(translations.input_charset), id(translations2.input_charset))
        tools.eq_(id(translations._catalog), id(translations2._catalog))

    def test_get_translation_object_optional_params(self):
        '''Smoketest leaving out optional parameters'''
        translations = i18n.get_translation_object('test')
        tools.assert_true(translations.__class__ in (i18n.NewGNUTranslations, i18n.DummyTranslations))

    def test_get_translation_object_python2_api_default(self):
        '''Smoketest that python2_api default value yields the python2 functions'''
        # Default
        translations = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)], codeset='utf-8')
        translations.input_charset = 'utf-8'
        tools.eq_(translations.gettext.__name__, '_gettext')
        tools.eq_(translations.lgettext.__name__, '_lgettext')
        tools.eq_(translations.ugettext.__name__, '_ugettext')
        tools.eq_(translations.ngettext.__name__, '_ngettext')
        tools.eq_(translations.lngettext.__name__, '_lngettext')
        tools.eq_(translations.ungettext.__name__, '_ungettext')

    def test_get_translation_object_python2_api_true(self):
        '''Smoketest that setting python2_api true yields the python2 functions'''
        # Default
        translations = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)], codeset='utf-8',
                python2_api=True)
        translations.input_charset = 'utf-8'
        tools.eq_(translations.gettext.__name__, '_gettext')
        tools.eq_(translations.lgettext.__name__, '_lgettext')
        tools.eq_(translations.ugettext.__name__, '_ugettext')
        tools.eq_(translations.ngettext.__name__, '_ngettext')
        tools.eq_(translations.lngettext.__name__, '_lngettext')
        tools.eq_(translations.ungettext.__name__, '_ungettext')

    def test_get_translation_object_python2_api_false(self):
        '''Smoketest that setting python2_api false yields the python3 functions'''
        # Default
        translations = i18n.get_translation_object('test',
                ['%s/data/locale' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)], codeset='utf-8',
                python2_api=False)
        translations.input_charset = 'utf-8'
        tools.eq_(translations.gettext.__name__, '_ugettext')
        tools.eq_(translations.lgettext.__name__, '_lgettext')
        tools.eq_(translations.ngettext.__name__, '_ungettext')
        tools.eq_(translations.lngettext.__name__, '_lngettext')

        tools.assert_raises(AttributeError, translations.ugettext, 'message')
        tools.assert_raises(AttributeError, translations.ungettext, 'message1', 'message2')

    def test_dummy_translation(self):
        '''Test that we can create a DummyTranslation object
        '''
        tools.assert_true(isinstance(i18n.DummyTranslations(), i18n.DummyTranslations))

# Note: Using nose's generator tests for this so we can't subclass
# unittest.TestCase
class TestDummyTranslations(base_classes.UnicodeTestData):
    def __init__(self):
        self.test_data = {'bytes': (( # First set is with default charset (utf8)
                (self.u_ascii, self.b_ascii),
                (self.u_spanish, self.utf8_spanish),
                (self.u_japanese, self.utf8_japanese),
                (self.b_ascii, self.b_ascii),
                (self.utf8_spanish, self.utf8_spanish),
                (self.latin1_spanish, self.utf8_mangled_spanish_latin1_as_utf8),
                (self.utf8_japanese, self.utf8_japanese),
                ),
                ( # Second set is with output_charset of latin1 (ISO-8859-1)
                (self.u_ascii, self.b_ascii),
                (self.u_spanish, self.latin1_spanish),
                (self.u_japanese, self.latin1_mangled_japanese_replace_as_latin1),
                (self.b_ascii, self.b_ascii),
                (self.utf8_spanish, self.utf8_spanish),
                (self.latin1_spanish, self.latin1_spanish),
                (self.utf8_japanese, self.utf8_japanese),
                ),
                ( # Third set is with output_charset of C
                (self.u_ascii, self.b_ascii),
                (self.u_spanish, self.ascii_mangled_spanish_as_ascii),
                (self.u_japanese, self.ascii_mangled_japanese_replace_as_latin1),
                (self.b_ascii, self.b_ascii),
                (self.utf8_spanish, self.ascii_mangled_spanish_as_ascii),
                (self.latin1_spanish, self.ascii_twice_mangled_spanish_latin1_as_utf8_as_ascii),
                (self.utf8_japanese, self.ascii_mangled_japanese_replace_as_latin1),
                ),
            ),
            'unicode': (( # First set is with the default charset (utf8)
                (self.u_ascii, self.u_ascii),
                (self.u_spanish, self.u_spanish),
                (self.u_japanese, self.u_japanese),
                (self.b_ascii, self.u_ascii),
                (self.utf8_spanish, self.u_spanish),
                (self.latin1_spanish, self.u_mangled_spanish_latin1_as_utf8), # String is mangled but no exception
                (self.utf8_japanese, self.u_japanese),
                ),
                ( # Second set is with _charset of latin1 (ISO-8859-1)
                (self.u_ascii, self.u_ascii),
                (self.u_spanish, self.u_spanish),
                (self.u_japanese, self.u_japanese),
                (self.b_ascii, self.u_ascii),
                (self.utf8_spanish, self.u_mangled_spanish_utf8_as_latin1), # String mangled but no exception
                (self.latin1_spanish, self.u_spanish),
                (self.utf8_japanese, self.u_mangled_japanese_utf8_as_latin1), # String mangled but no exception
                ),
                ( # Third set is with _charset of C
                (self.u_ascii, self.u_ascii),
                (self.u_spanish, self.u_spanish),
                (self.u_japanese, self.u_japanese),
                (self.b_ascii, self.u_ascii),
                (self.utf8_spanish, self.u_mangled_spanish_utf8_as_ascii), # String mangled but no exception
                (self.latin1_spanish, self.u_mangled_spanish_latin1_as_ascii), # String mangled but no exception
                (self.utf8_japanese, self.u_mangled_japanese_utf8_as_ascii), # String mangled but no exception
                ),
            )
    }

    def setUp(self):
        self.translations = i18n.DummyTranslations()

    def check_gettext(self, message, value, charset=None):
        self.translations.set_output_charset(charset)
        tools.eq_(self.translations.gettext(message), value,
                msg='gettext(%s): trans: %s != val: %s (charset=%s)'
                % (repr(message), repr(self.translations.gettext(message)),
                    repr(value), charset))

    def check_lgettext(self, message, value, charset=None,
            locale='en_US.UTF-8'):
        os.environ['LC_ALL'] = locale
        self.translations.set_output_charset(charset)
        tools.eq_(self.translations.lgettext(message), value,
                msg='lgettext(%s): trans: %s != val: %s (charset=%s, locale=%s)'
                % (repr(message), repr(self.translations.lgettext(message)),
                    repr(value), charset, locale))

    # Note: charset has a default value because nose isn't invoking setUp and
    # tearDown each time check_* is run.
    def check_ugettext(self, message, value, charset='utf-8'):
        '''ugettext method with default values'''
        self.translations.input_charset = charset
        tools.eq_(self.translations.ugettext(message), value,
                msg='ugettext(%s): trans: %s != val: %s (charset=%s)'
                % (repr(message), repr(self.translations.ugettext(message)),
                    repr(value), charset))

    def check_ngettext(self, message, value, charset=None):
        self.translations.set_output_charset(charset)
        tools.eq_(self.translations.ngettext(message, 'blank', 1), value)
        tools.eq_(self.translations.ngettext('blank', message, 2), value)
        tools.assert_not_equal(self.translations.ngettext(message, 'blank', 2), value)
        tools.assert_not_equal(self.translations.ngettext('blank', message, 1), value)

    def check_lngettext(self, message, value, charset=None, locale='en_US.UTF-8'):
        os.environ['LC_ALL'] = locale
        self.translations.set_output_charset(charset)
        tools.eq_(self.translations.lngettext(message, 'blank', 1), value,
                msg='lngettext(%s, "blank", 1): trans: %s != val: %s (charset=%s, locale=%s)'
                % (repr(message), repr(self.translations.lngettext(message,
                    'blank', 1)), repr(value), charset, locale))
        tools.eq_(self.translations.lngettext('blank', message, 2), value,
                msg='lngettext("blank", %s, 2): trans: %s != val: %s (charset=%s, locale=%s)'
                % (repr(message), repr(self.translations.lngettext('blank',
                    message, 2)), repr(value), charset, locale))
        tools.assert_not_equal(self.translations.lngettext(message, 'blank', 2), value,
                msg='lngettext(%s, "blank", 2): trans: %s, val: %s (charset=%s, locale=%s)'
                % (repr(message), repr(self.translations.lngettext(message,
                    'blank', 2)), repr(value), charset, locale))
        tools.assert_not_equal(self.translations.lngettext('blank', message, 1), value,
                msg='lngettext("blank", %s, 1): trans: %s != val: %s (charset=%s, locale=%s)'
                % (repr(message), repr(self.translations.lngettext('blank',
                    message, 1)), repr(value), charset, locale))

    # Note: charset has a default value because nose isn't invoking setUp and
    # tearDown each time check_* is run.
    def check_ungettext(self, message, value, charset='utf-8'):
        self.translations.input_charset = charset
        tools.eq_(self.translations.ungettext(message, 'blank', 1), value)
        tools.eq_(self.translations.ungettext('blank', message, 2), value)
        tools.assert_not_equal(self.translations.ungettext(message, 'blank', 2), value)
        tools.assert_not_equal(self.translations.ungettext('blank', message, 1), value)

    def test_gettext(self):
        '''gettext method with default values'''
        for message, value in self.test_data['bytes'][0]:
            yield self.check_gettext, message, value

    def test_gettext_output_charset(self):
        '''gettext method after output_charset is set'''
        for message, value in self.test_data['bytes'][1]:
            yield self.check_gettext, message, value, 'latin1'

    def test_ngettext(self):
        for message, value in self.test_data['bytes'][0]:
            yield self.check_ngettext, message, value

    def test_ngettext_output_charset(self):
        for message, value in self.test_data['bytes'][1]:
            yield self.check_ngettext, message, value, 'latin1'

    def test_lgettext(self):
        '''lgettext method with default values on a utf8 locale'''
        for message, value in self.test_data['bytes'][0]:
            yield self.check_lgettext, message, value

    def test_lgettext_output_charset(self):
        '''lgettext method after output_charset is set'''
        for message, value in self.test_data['bytes'][1]:
            yield self.check_lgettext, message, value, 'latin1'

    def test_lgettext_output_charset_and_locale(self):
        '''lgettext method after output_charset is set in C locale

        output_charset should take precedence
        '''
        for message, value in self.test_data['bytes'][1]:
            yield self.check_lgettext, message, value, 'latin1', 'C'

    def test_lgettext_locale_C(self):
        '''lgettext method in a C locale'''
        for message, value in self.test_data['bytes'][2]:
            yield self.check_lgettext, message, value, None, 'C'

    def test_lngettext(self):
        '''lngettext method with default values on a utf8 locale'''
        for message, value in self.test_data['bytes'][0]:
            yield self.check_lngettext, message, value

    def test_lngettext_output_charset(self):
        '''lngettext method after output_charset is set'''
        for message, value in self.test_data['bytes'][1]:
            yield self.check_lngettext, message, value, 'latin1'

    def test_lngettext_output_charset_and_locale(self):
        '''lngettext method after output_charset is set in C locale

        output_charset should take precedence
        '''
        for message, value in self.test_data['bytes'][1]:
            yield self.check_lngettext, message, value, 'latin1', 'C'

    def test_lngettext_locale_C(self):
        '''lngettext method in a C locale'''
        for message, value in self.test_data['bytes'][2]:
            yield self.check_lngettext, message, value, None, 'C'

    def test_ugettext(self):
        for message, value in self.test_data['unicode'][0]:
            yield self.check_ugettext, message, value

    def test_ugettext_charset_latin1(self):
        for message, value in self.test_data['unicode'][1]:
            yield self.check_ugettext, message, value, 'latin1'

    def test_ugettext_charset_ascii(self):
        for message, value in self.test_data['unicode'][2]:
            yield self.check_ugettext, message, value, 'ascii'

    def test_ungettext(self):
        for message, value in self.test_data['unicode'][0]:
            yield self.check_ungettext, message, value

    def test_ungettext_charset_latin1(self):
        for message, value in self.test_data['unicode'][1]:
            yield self.check_ungettext, message, value, 'latin1'

    def test_ungettext_charset_ascii(self):
        for message, value in self.test_data['unicode'][2]:
            yield self.check_ungettext, message, value, 'ascii'

    def test_nonbasestring(self):
        tools.eq_(self.translations.gettext(dict(hi='there')), self.b_empty_string)
        tools.eq_(self.translations.ngettext(dict(hi='there'), dict(hi='two'), 1), self.b_empty_string)
        tools.eq_(self.translations.lgettext(dict(hi='there')), self.b_empty_string)
        tools.eq_(self.translations.lngettext(dict(hi='there'), dict(hi='two'), 1), self.b_empty_string)
        tools.eq_(self.translations.ugettext(dict(hi='there')), self.u_empty_string)
        tools.eq_(self.translations.ungettext(dict(hi='there'), dict(hi='two'), 1), self.u_empty_string)


class TestI18N_Latin1(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.iso88591'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_easy_gettext_setup_non_unicode(self):
        '''Test that the easy_gettext_setup function works
        '''
        b_, bN_ = i18n.easy_gettext_setup('foo', localedirs=
                ['%s/data/locale/' % os.path.dirname(__file__)],
                use_unicode=False)

        tools.eq_(b_(self.utf8_spanish), self.utf8_spanish)
        tools.eq_(b_(self.u_spanish), self.latin1_spanish)
        tools.eq_(bN_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_limao)
        tools.eq_(bN_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_limoes)
        tools.eq_(bN_(self.u_limao, self.u_limoes, 1), self.latin1_limao)
        tools.eq_(bN_(self.u_limao, self.u_limoes, 2), self.latin1_limoes)


class TestNewGNUTranslationsNoMatch(TestDummyTranslations):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test', ['%s/data/locale/' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE


class TestNewGNURealTranslations_UTF8(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test', ['%s/data/locale/' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_gettext(self):
        _ = self.translations.gettext
        tools.eq_(_(self.utf8_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.utf8_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.utf8_in_fallback), self.utf8_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.utf8_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.u_in_fallback), self.utf8_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.utf8_not_in_catalog)

    def test_ngettext(self):
        _ = self.translations.ngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.utf8_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.utf8_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)


    def test_lgettext(self):
        _ = self.translations.lgettext
        tools.eq_(_(self.utf8_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.utf8_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.utf8_in_fallback), self.utf8_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.utf8_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.u_in_fallback), self.utf8_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.utf8_not_in_catalog)

    def test_lngettext(self):
        _ = self.translations.lngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.utf8_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.utf8_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)


    def test_ugettext(self):
        _ = self.translations.ugettext
        tools.eq_(_(self.utf8_kitchen), self.u_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.u_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.u_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.utf8_in_fallback), self.u_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.u_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.u_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.u_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.u_ja_kuratomi)
        # This is not translated to utf8_yes_in_fallback because this test is
        # without the fallback message catalog
        tools.eq_(_(self.u_in_fallback), self.u_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.u_not_in_catalog)

    def test_ungettext(self):
        _ = self.translations.ungettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.u_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.u_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.u_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.u_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.u_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.u_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.u_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.u_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.u_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.u_not_in_catalog)


class TestNewGNURealTranslations_Latin1(TestNewGNURealTranslations_UTF8):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.iso88591'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test', ['%s/data/locale/' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_lgettext(self):
        _ = self.translations.lgettext
        tools.eq_(_(self.utf8_kitchen), self.latin1_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.latin1_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.latin1_ja_kuratomi)
        # Neither of the following two tests encode to proper latin-1 because:
        # any byte is valid in latin-1 so there's no way to know that what
        # we're given in the string is really utf-8
        #
        # This is not translated to latin1_yes_in_fallback because this test
        # is without the fallback message catalog
        tools.eq_(_(self.utf8_in_fallback), self.utf8_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.latin1_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.latin1_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.latin1_ja_kuratomi)
        # This is not translated to latin1_yes_in_fallback because this test
        # is without the fallback message catalog
        tools.eq_(_(self.u_in_fallback), self.latin1_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.latin1_not_in_catalog)

    def test_lngettext(self):
        _ = self.translations.lngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.latin1_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.latin1_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.latin1_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.latin1_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.latin1_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.latin1_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.latin1_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.latin1_lemons)

        # This unfortunately does not encode to proper latin-1 because:
        # any byte is valid in latin-1 so there's no way to know that what
        # we're given in the string is really utf-8
        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.latin1_not_in_catalog)


class TestFallbackNewGNUTranslationsNoMatch(TestDummyTranslations):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test',
                ['%s/data/locale/' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE


class TestFallbackNewGNURealTranslations_UTF8(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test',
                ['%s/data/locale/' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_gettext(self):
        _ = self.translations.gettext
        tools.eq_(_(self.utf8_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.utf8_in_fallback), self.utf8_yes_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.u_in_fallback), self.utf8_yes_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.utf8_not_in_catalog)

    def test_ngettext(self):
        _ = self.translations.ngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.utf8_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.utf8_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)

    def test_lgettext(self):
        _ = self.translations.lgettext
        tools.eq_(_(self.utf8_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.utf8_in_fallback), self.utf8_yes_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.utf8_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.u_in_fallback), self.utf8_yes_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.utf8_not_in_catalog)

    def test_lngettext(self):
        _ = self.translations.lngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.utf8_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.utf8_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.utf8_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.utf8_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.utf8_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.utf8_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)

    def test_ugettext(self):
        _ = self.translations.ugettext
        tools.eq_(_(self.utf8_kitchen), self.u_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.u_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.u_ja_kuratomi)
        tools.eq_(_(self.utf8_in_fallback), self.u_yes_in_fallback)
        tools.eq_(_(self.utf8_not_in_catalog), self.u_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.u_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.u_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.u_ja_kuratomi)
        tools.eq_(_(self.u_in_fallback), self.u_yes_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.u_not_in_catalog)

    def test_ungettext(self):
        _ = self.translations.ungettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.u_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.u_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.u_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.u_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.u_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.u_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.u_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.u_lemons)

        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.u_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.u_not_in_catalog)


class TestFallbackNewGNURealTranslations_Latin1(TestFallbackNewGNURealTranslations_UTF8):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.iso88591'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.translations = i18n.get_translation_object('test',
                ['%s/data/locale/' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_lgettext(self):
        _ = self.translations.lgettext
        tools.eq_(_(self.utf8_kitchen), self.latin1_pt_kitchen)
        tools.eq_(_(self.utf8_ja_kuratomi), self.latin1_kuratomi)
        tools.eq_(_(self.utf8_kuratomi), self.latin1_ja_kuratomi)
        tools.eq_(_(self.utf8_in_fallback), self.latin1_yes_in_fallback)
        # This unfortunately does not encode to proper latin-1 because:
        # any byte is valid in latin-1 so there's no way to know that what
        # we're given in the string is really utf-8
        tools.eq_(_(self.utf8_not_in_catalog), self.utf8_not_in_catalog)

        tools.eq_(_(self.u_kitchen), self.latin1_pt_kitchen)
        tools.eq_(_(self.u_ja_kuratomi), self.latin1_kuratomi)
        tools.eq_(_(self.u_kuratomi), self.latin1_ja_kuratomi)
        tools.eq_(_(self.u_in_fallback), self.latin1_yes_in_fallback)
        tools.eq_(_(self.u_not_in_catalog), self.latin1_not_in_catalog)

    def test_lngettext(self):
        _ = self.translations.lngettext
        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 1), self.latin1_limao)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 1), self.latin1_lemon)
        tools.eq_(_(self.u_lemon, self.u_lemons, 1), self.latin1_limao)
        tools.eq_(_(self.u_limao, self.u_limoes, 1), self.latin1_lemon)

        tools.eq_(_(self.utf8_lemon, self.utf8_lemons, 2), self.latin1_limoes)
        tools.eq_(_(self.utf8_limao, self.utf8_limoes, 2), self.latin1_lemons)
        tools.eq_(_(self.u_lemon, self.u_lemons, 2), self.latin1_limoes)
        tools.eq_(_(self.u_limao, self.u_limoes, 2), self.latin1_lemons)

        # This unfortunately does not encode to proper latin-1 because:
        # any byte is valid in latin-1 so there's no way to know that what
        # we're given in the string is really utf-8
        tools.eq_(_(self.utf8_not_in_catalog, 'throwaway', 1), self.utf8_not_in_catalog)
        tools.eq_(_(self.u_not_in_catalog, 'throwaway', 1), self.latin1_not_in_catalog)


class TestFallback(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.iso88591'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.gtranslations = i18n.get_translation_object('test',
                ['%s/data/locale/' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])
        self.gtranslations.add_fallback(object())
        self.dtranslations = i18n.get_translation_object('nonexistent',
                ['%s/data/locale/' % os.path.dirname(__file__),
                    '%s/data/locale-old' % os.path.dirname(__file__)])
        self.dtranslations.add_fallback(object())


    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

    def test_invalid_fallback_no_raise(self):
        '''Test when we have an invalid fallback that it does not raise.'''
        tools.eq_(self.gtranslations.gettext(self.u_spanish), self.utf8_spanish)
        tools.eq_(self.gtranslations.ugettext(self.u_spanish), self.u_spanish)
        tools.eq_(self.gtranslations.lgettext(self.u_spanish), self.latin1_spanish)

        tools.eq_(self.gtranslations.ngettext(self.u_spanish, 'cde', 1), self.utf8_spanish)
        tools.eq_(self.gtranslations.ungettext(self.u_spanish, 'cde', 1), self.u_spanish)
        tools.eq_(self.gtranslations.lngettext(self.u_spanish, 'cde', 1), self.latin1_spanish)

        tools.eq_(self.dtranslations.gettext(self.u_spanish), self.utf8_spanish)
        tools.eq_(self.dtranslations.ugettext(self.u_spanish), self.u_spanish)
        tools.eq_(self.dtranslations.lgettext(self.u_spanish), self.latin1_spanish)

        tools.eq_(self.dtranslations.ngettext(self.u_spanish, 'cde', 1), self.utf8_spanish)
        tools.eq_(self.dtranslations.ungettext(self.u_spanish, 'cde', 1), self.u_spanish)
        tools.eq_(self.dtranslations.lngettext(self.u_spanish, 'cde', 1), self.latin1_spanish)


class TestDefaultLocaleDir(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.old_LC_ALL = os.environ.get('LC_ALL', None)
        os.environ['LC_ALL'] = 'pt_BR.utf8'

        self.old_LANGUAGE = os.environ.pop('LANGUAGE', None)

        self.old_DEFAULT_LOCALEDIRS = i18n._DEFAULT_LOCALEDIR
        i18n._DEFAULT_LOCALEDIR = '%s/data/locale/' % os.path.dirname(__file__)

        self.translations = i18n.get_translation_object('test')

    def tearDown(self):
        if self.old_LC_ALL:
            os.environ['LC_ALL'] = self.old_LC_ALL
        else:
            del(os.environ['LC_ALL'])

        if self.old_LANGUAGE:
            os.environ['LANGUAGE'] = self.old_LANGUAGE

        if self.old_DEFAULT_LOCALEDIRS:
            i18n._DEFAULT_LOCALEDIR = self.old_DEFAULT_LOCALEDIRS

    def test_gettext(self):
        _ = self.translations.gettext
        tools.eq_(_(self.utf8_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.utf8_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.utf8_ja_kuratomi), self.utf8_kuratomi)
        # Returns msgid because the string is in a fallback catalog which we
        # haven't setup
        tools.eq_(_(self.utf8_in_fallback), self.utf8_in_fallback)

        tools.eq_(_(self.u_kitchen), self.utf8_pt_kitchen)
        tools.eq_(_(self.u_kuratomi), self.utf8_ja_kuratomi)
        tools.eq_(_(self.u_ja_kuratomi), self.utf8_kuratomi)
        # Returns msgid because the string is in a fallback catalog which we
        # haven't setup
        tools.eq_(_(self.u_in_fallback), self.utf8_in_fallback)



from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 1, 0),)
__version__ = version_tuple_to_string(__version_info__)

from kitchen.collections import strictdict
from kitchen.collections.strictdict import StrictDict

__all__ = ('strictdict', 'StrictDict',)



# -*- coding: utf-8 -*-
#
# Base class for testing unicode and utf8 functions.  This holds data that's
# useful for making tests

import re

from kitchen.text.converters import to_bytes
from kitchen.text import misc

class UnicodeTestData(object):
    u_empty_string = ''
    b_empty_string = b''
    # This should encode fine -- sanity check
    u_ascii = 'the quick brown fox jumped over the lazy dog'
    b_ascii = b'the quick brown fox jumped over the lazy dog'

    # First challenge -- what happens with latin-1 characters
    u_spanish = 'El veloz murciélago saltó sobre el perro perezoso.'
    # utf8 and latin1 both support these chars so no mangling
    utf8_spanish = u_spanish.encode('utf8')
    latin1_spanish = u_spanish.encode('latin1')

    # ASCII does not have the accented characters so it mangles
    ascii_mangled_spanish_as_ascii = u_spanish.encode('ascii', 'replace')
    # Attempting to decode using the wrong charset will mangle
    # Note: as a general principle, we do not want to have code that mangles
    # input of one charset and output of the same charset.  We want to avoid
    # things like::
    #   input latin-1, transform to unicode with utf-8, output latin-1.
    u_mangled_spanish_utf8_as_latin1 = str(utf8_spanish, encoding='latin1', errors='replace')
    u_mangled_spanish_utf8_as_ascii = str(utf8_spanish, encoding='ascii', errors='replace')
    u_mangled_spanish_latin1_as_ascii = str(latin1_spanish, encoding='ascii', errors='replace')
    u_mangled_spanish_latin1_as_utf8 = str(latin1_spanish, encoding='utf-8', errors='replace')
    ascii_twice_mangled_spanish_latin1_as_utf8_as_ascii = u_mangled_spanish_latin1_as_utf8.encode('ascii', 'replace')
    utf8_mangled_spanish_latin1_as_utf8 = u_mangled_spanish_latin1_as_utf8.encode('utf-8')
    u_spanish_ignore = str(latin1_spanish, encoding='utf8', errors='ignore')

    u_japanese = "速い茶色のキツネが怠惰な犬に'増"
    utf8_japanese = u_japanese.encode('utf8')
    euc_jp_japanese = u_japanese.encode('euc_jp')
    u_mangled_euc_jp_as_latin1 = str(euc_jp_japanese, 'latin1')
    u_mangled_euc_jp_as_utf8 = str(euc_jp_japanese, 'utf-8', 'replace')
    utf8_mangled_euc_jp_as_latin1 = u_mangled_euc_jp_as_latin1.encode('utf8')
    u_mangled_japanese_utf8_as_latin1 = str(utf8_japanese, 'latin1')
    u_mangled_japanese_utf8_as_ascii = "������������������������������������������'���"
    ascii_mangled_japanese_replace_as_latin1 = b"??????????????'?"
    latin1_mangled_japanese_replace_as_latin1 = b"??????????????'?"

    u_mixed = 'く ku ら ra と to み mi'
    utf8_mixed = u_mixed.encode('utf8')
    utf8_ku = u_mixed[0].encode('utf8')
    utf8_ra = u_mixed[2].encode('utf8')
    utf8_to = u_mixed[4].encode('utf8')
    utf8_mi = u_mixed[6].encode('utf8')

    u_mixed_replace = '\ufffd ku \ufffd ra \ufffd to \ufffd mi'
    u_mixed_ignore = ' ku  ra  to  mi'
    latin1_mixed_replace = b'? ku ? ra ? to ? mi'
    latin1_mixed_ignore = b' ku  ra  to  mi'

    u_entity = 'Test: <"&"> – ' + u_japanese + 'é'
    utf8_entity = u_entity.encode('utf8')
    u_entity_escape = 'Test: &lt;&quot;&amp;&quot;&gt; &ndash; ' + str(u_japanese.encode('ascii', 'xmlcharrefreplace'), 'ascii') + '&#xe9;'
    utf8_entity_escape = b'Test: &lt;"&amp;"&gt; ' + ('– ' + u_japanese + 'é').encode('utf8')
    utf8_attrib_escape = b'Test: &lt;&quot;&amp;&quot;&gt; ' + ('– ' + u_japanese + 'é').encode('utf8')
    ascii_entity_escape = (b'Test: <"&"> '.replace(b'&', b'&amp;', 1).replace(b'<', b'&lt;').replace(b'>', b'&gt;')) + ('– ' + u_japanese + 'é').encode('ascii', 'xmlcharrefreplace')
    ascii_attrib_escape = (b'Test: <"&"> '.replace(b'&', b'&amp;', 1).replace(b'<', b'&lt;').replace(b'>', b'&gt;').replace(b'"', b'&quot;')) + ('– ' + u_japanese + 'é').encode('ascii', 'xmlcharrefreplace')

    b_byte_chars = b' '.join((bytes((b,)) for b in range(0, 256)))
    b_byte_encoded = b'ACABIAIgAyAEIAUgBiAHIAggCSAKIAsgDCANIA4gDyAQIBEgEiATIBQgFSAWIBcgGCAZIBogGyAcIB0gHiAfICAgISAiICMgJCAlICYgJyAoICkgKiArICwgLSAuIC8gMCAxIDIgMyA0IDUgNiA3IDggOSA6IDsgPCA9ID4gPyBAIEEgQiBDIEQgRSBGIEcgSCBJIEogSyBMIE0gTiBPIFAgUSBSIFMgVCBVIFYgVyBYIFkgWiBbIFwgXSBeIF8gYCBhIGIgYyBkIGUgZiBnIGggaSBqIGsgbCBtIG4gbyBwIHEgciBzIHQgdSB2IHcgeCB5IHogeyB8IH0gfiB/IIAggSCCIIMghCCFIIYghyCIIIkgiiCLIIwgjSCOII8gkCCRIJIgkyCUIJUgliCXIJggmSCaIJsgnCCdIJ4gnyCgIKEgoiCjIKQgpSCmIKcgqCCpIKogqyCsIK0griCvILAgsSCyILMgtCC1ILYgtyC4ILkguiC7ILwgvSC+IL8gwCDBIMIgwyDEIMUgxiDHIMggySDKIMsgzCDNIM4gzyDQINEg0iDTINQg1SDWINcg2CDZINog2yDcIN0g3iDfIOAg4SDiIOMg5CDlIOYg5yDoIOkg6iDrIOwg7SDuIO8g8CDxIPIg8yD0IPUg9iD3IPgg+SD6IPsg/CD9IP4g/w=='

    repr_re = re.compile(b'^<[^ ]*\.([^.]+) object at .*>$')

    u_paragraph = '''ConfigObj is a simple but powerful config file reader and writer: an ini file
round tripper. Its main feature is that it is very easy to use, with a
straightforward programmer's interface and a simple syntax for config files.
It has lots of other features though:



    * Nested sections (subsections), to any level
    * List values
    * Multiple line values
    * String interpolation (substitution)
    * Integrated with a powerful validation system
          o including automatic type checking/conversion
          o repeated sections
          o and allowing default values
    * All comments in the file are preserved
    * The order of keys/sections is preserved
    * No external dependencies
    * Full Unicode support
    * A powerful unrepr mode for storing basic datatypes
'''
    utf8_paragraph = u_paragraph.encode('utf-8', 'replace')
    u_paragraph_out = ['ConfigObj is a simple but powerful config file reader and writer: an',
'ini file round tripper. Its main feature is that it is very easy to',
"use, with a straightforward programmer's interface and a simple syntax",
'for config files. It has lots of other features though:',
'',
'',
'',
'    * Nested sections (subsections), to any level',
'    * List values',
'    * Multiple line values',
'    * String interpolation (substitution)',
'    * Integrated with a powerful validation system',
'          o including automatic type checking/conversion',
'          o repeated sections',
'          o and allowing default values',
'    * All comments in the file are preserved',
'    * The order of keys/sections is preserved',
'    * No external dependencies',
'    * Full Unicode support',
'    * A powerful unrepr mode for storing basic datatypes']

    utf8_paragraph_out = [line.encode('utf-8', 'replace') for line in u_paragraph_out]

    u_mixed_para = 'くらとみ kuratomi ' * 5
    utf8_mixed_para = u_mixed_para.encode('utf8')
    u_mixed_para_out = ['くらとみ kuratomi くらとみ kuratomi くらとみ kuratomi くらとみ',
            'kuratomi くらとみ kuratomi']
    u_mixed_para_57_initial_subsequent_out = ['    くらとみ kuratomi くらとみ kuratomi くらとみ kuratomi',
        '----くらとみ kuratomi くらとみ kuratomi']
    utf8_mixed_para_out = list(map(to_bytes, u_mixed_para_out))
    utf8_mixed_para_57_initial_subsequent_out = list(map(to_bytes, u_mixed_para_57_initial_subsequent_out))

    u_ascii_chars = ' '.join(map(chr, list(range(0, 256))))
    u_ascii_no_ctrl = ''.join([c for c in u_ascii_chars if ord(c) not in misc._CONTROL_CODES])
    u_ascii_ctrl_replace = u_ascii_chars.translate(dict([(c, '?') for c in misc._CONTROL_CODES]))
    utf8_ascii_chars = u_ascii_chars.encode('utf8')

    # These are present in the test catalog as msgids or values
    u_lemon = '1 lemon'
    utf8_lemon = u_lemon.encode('utf-8')
    latin1_lemon = u_lemon.encode('latin-1')

    u_lemons = '4 lemons'
    utf8_lemons = u_lemons.encode('utf-8')
    latin1_lemons = u_lemons.encode('latin-1')

    u_limao = '一 limão'
    utf8_limao = u_limao.encode('utf-8')
    latin1_limao = u_limao.encode('latin-1', 'replace')

    u_limoes = '四 limões'
    utf8_limoes = u_limoes.encode('utf-8')
    latin1_limoes = u_limoes.encode('latin-1', 'replace')

    u_not_in_catalog = 'café not matched in catalogs'
    utf8_not_in_catalog = u_not_in_catalog.encode('utf-8')
    latin1_not_in_catalog = u_not_in_catalog.encode('latin-1')

    u_kitchen = 'kitchen sink'
    utf8_kitchen = u_kitchen.encode('utf-8')
    latin1_kitchen = u_kitchen.encode('latin-1')

    u_pt_kitchen = 'pia da cozinha'
    utf8_pt_kitchen = u_pt_kitchen.encode('utf-8')
    latin1_pt_kitchen = u_pt_kitchen.encode('latin-1')

    u_kuratomi = 'Kuratomi'
    utf8_kuratomi = u_kuratomi.encode('utf-8')
    latin1_kuratomi = u_kuratomi.encode('latin-1')

    u_ja_kuratomi = 'くらとみ'
    utf8_ja_kuratomi = u_ja_kuratomi.encode('utf-8')
    latin1_ja_kuratomi = u_ja_kuratomi.encode('latin-1', 'replace')

    u_in_fallback = 'Only café in fallback'
    utf8_in_fallback = u_in_fallback.encode('utf-8')
    latin1_in_fallback = u_in_fallback.encode('latin-1')

    u_yes_in_fallback = 'Yes, only café in fallback'
    utf8_yes_in_fallback = u_yes_in_fallback.encode('utf-8')
    latin1_yes_in_fallback = u_yes_in_fallback.encode('latin-1')



# -*- coding: utf-8 -*-
#
import unittest
from nose import tools

from kitchen.pycompat24.sets import add_builtin_set
add_builtin_set()

from kitchen import collections

def test_strict_dict_get_set():
    '''Test getting and setting items in StrictDict'''
    d = collections.StrictDict()
    d['a'] = 1
    d[b'a'] = 2
    tools.assert_not_equal(d['a'], d[b'a'])
    tools.eq_(len(d), 2)

    d['\xf1'] = 1
    d[b'\xf1'] = 2
    d['\xf1'.encode('utf-8')] = 3
    tools.eq_(d['\xf1'], 1)
    tools.eq_(d[b'\xf1'], 2)
    tools.eq_(d['\xf1'.encode('utf-8')], 3)
    tools.eq_(len(d), 5)

class TestStrictDict(unittest.TestCase):
    def setUp(self):
        self.d = collections.StrictDict()
        self.d['a'] = 1
        self.d[b'a'] = 2
        self.d['\xf1'] = 1
        self.d[b'\xf1'] = 2
        self.d['\xf1'.encode('utf-8')] = 3
        self.keys = ['a', b'a', '\xf1', b'\xf1', '\xf1'.encode('utf-8')]

    def tearDown(self):
        del(self.d)

    def _compare_lists(self, list1, list2, debug=False):
        '''We have a mixture of bytes and unicode so we have to compare these
            lists manually and inefficiently
        '''
        def _compare_lists_helper(compare_to, dupes, idx, length):
            if i not in compare_to:
                return False
            for n in range(1, length + 1):
                if i not in dupes[n][idx]:
                    dupes[n][idx].add(i)
                    return True
        if len(list1) != len(list2):
            return False

        list1_dupes = dict([(i, (set(), set(), set())) for i in range(1, len(list1)+1)])
        list2_dupes = dict([(i, (set(), set(), set())) for i in range(1, len(list1)+1)])

        list1_u = [l for l in list1 if isinstance(l, str)]
        list1_b = [l for l in list1 if isinstance(l, bytes)]
        list1_o = [l for l in list1 if not (isinstance(l, (str, bytes)))]

        list2_u = [l for l in list2 if isinstance(l, str)]
        list2_b = [l for l in list2 if isinstance(l, bytes)]
        list2_o = [l for l in list2 if not (isinstance(l, (str, bytes)))]

        for i in list1:
            if isinstance(i, str):
                if not _compare_lists_helper(list2_u, list1_dupes, 0, len(list1)):
                    return False
            elif isinstance(i, bytes):
                if not _compare_lists_helper(list2_b, list1_dupes, 1, len(list1)):
                    return False
            else:
                if not _compare_lists_helper(list2_o, list1_dupes, 2, len(list1)):
                    return False

        if list1_dupes[2][0] or list1_dupes[2][1] or list1_dupes[2][2]:
            for i in list2:
                if isinstance(i, str):
                    if not _compare_lists_helper(list1_u, list2_dupes, 0, len(list1)):
                        return False
                elif isinstance(i, bytes):
                    if not _compare_lists_helper(list1_b, list2_dupes, 1, len(list1)):
                        return False
                else:
                    if not _compare_lists_helper(list1_o, list2_dupes, 2, len(list1)):
                        return False

            for i in range(2, len(list1)+1):
                for n in list1_dupes[i]:
                    if n not in list2_dupes[i]:
                        return False

        return True

    def test__compare_list(self):
        '''*sigh* this test support function is so complex we need to test it'''
        tools.ok_(self._compare_lists(['a', 'b', 'c'], ['c', 'a', 'b']))
        tools.ok_(not self._compare_lists(['b', 'c'], ['c', 'a', 'b']))
        tools.ok_(not self._compare_lists([b'a', 'b'], ['a', 'b']))
        tools.ok_(not self._compare_lists(['a', b'b'], [b'a', 'b']))
        tools.ok_(self._compare_lists(['a', 'b', 1], ['a', 1, 'b']))
        tools.ok_(self._compare_lists([b'a', b'b'], [b'a', b'b']))
        tools.ok_(self._compare_lists([b'a', 'b'], [b'a', 'b']))
        tools.ok_(not self._compare_lists([b'a', 'b'], [b'a', b'b']))
        tools.ok_(self._compare_lists([b'a', 'b', 'b', 'c', b'a'], [b'a', b'a', 'b', 'c', 'b']))
        tools.ok_(not self._compare_lists([b'a', 'b', 'b', 'c', 'a'], [b'a', b'a', 'b', 'c', 'b']))
        tools.ok_(not self._compare_lists([b'a', 'b', 'b', 'c', b'a'], [b'a', 'b', 'b', 'c', 'b']))

    def test_strict_dict_len(self):
        '''StrictDict len'''
        tools.eq_(len(self.d), 5)

    def test_strict_dict_del(self):
        '''StrictDict del'''
        tools.eq_(len(self.d), 5)
        del(self.d['\xf1'])
        tools.assert_raises(KeyError, self.d.__getitem__, '\xf1')
        tools.eq_(len(self.d), 4)

    def test_strict_dict_iter(self):
        '''StrictDict iteration'''
        keys = []
        for k in self.d:
            keys.append(k)
        tools.ok_(self._compare_lists(keys, self.keys),
                msg='keys != self.key: %s != %s' % (keys, self.keys))

        keys = []
        for k in self.d.keys():
            keys.append(k)
        tools.ok_(self._compare_lists(keys, self.keys),
                msg='keys != self.key: %s != %s' % (keys, self.keys))

        keys = [k for k in self.d]
        tools.ok_(self._compare_lists(keys, self.keys),
                msg='keys != self.key: %s != %s' % (keys, self.keys))

        keys = []
        for k in list(self.d.keys()):
            keys.append(k)
        tools.ok_(self._compare_lists(keys, self.keys),
                msg='keys != self.key: %s != %s' % (keys, self.keys))

    def test_strict_dict_contains(self):
        '''StrictDict contains function'''
        tools.ok_(b'b' not in self.d)
        tools.ok_('b' not in self.d)
        tools.ok_(b'\xf1' in self.d)
        tools.ok_('\xf1' in self.d)
        tools.ok_(b'a' in self.d)
        tools.ok_('a' in self.d)

        del(self.d['\xf1'])
        tools.ok_('\xf1' not in self.d)
        tools.ok_(b'\xf1' in self.d)

        del(self.d[b'a'])
        tools.ok_('a' in self.d)
        tools.ok_(b'a' not in self.d)



'''
The :mod:`kitchen.pycompat24` module contains implementations of functionality
introduced in python-2.4 for use on earlier versions of python.
'''
from kitchen.versioning import version_tuple_to_string

__version_info__ = ((1, 1, 0),)
__version__ = version_tuple_to_string(__version_info__)

__all__ = ('base64', 'sets', 'subprocess')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Red Hat, Inc
#
# This file is part of kitchen
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>

'''
Implement the modern subprocess interface

Python-2.5 and python-2.7 introduce new API features to subprocess.  This is
a backport of that module for use on earlier python versions.

.. seealso::
    :mod:`subprocess`
        for information about using the functions provided here.
'''
import warnings

warnings.warn('In python3, kitchen.pycompat27.subprocess is deprecated.'
        '  If your code doesn\'t have to remain compatible with python less'
        ' than 2.7 use subprocess from the python3 stdlib',
        PendingDeprecationWarning, stacklevel=2)

# :W0401,W0611,W0614: We're importing compatibility to the python-2.7 version
#   of subprocess.
#pylint:disable-msg=W0401,W0611,W0614
# All versions of python3 have a modern enough subprocess.  This module only
# exists for backwards compatibility
from subprocess import *
from subprocess import list2cmdline
from subprocess import __all__

# subprocess.MAXFD was removed in python-3.5
# https://github.com/fedora-infra/kitchen/issues/10
try:
    from subprocess import MAXFD
except ImportError:
    try:
        import os
        MAXFD = os.sysconf("SC_OPEN_MAX")
    except ValueError:
        MAXFD = 256

# subprocess.mswindows was renamed in python-3.5
# https://github.com/fedora-infra/kitchen/issues/12
try:
    from subprocess import mswindows
except ImportError:
    # Python 3.5
    from subprocess import _mswindows as mswindows



# -*- coding: utf-8 -*-
#

import unittest
from nose import tools
from nose.plugins.skip import SkipTest

import io
import sys
import warnings

try:
    import chardet
except:
    chardet = None

from kitchen.text import converters
from kitchen.text.exceptions import XmlEncodeError

import base_classes

class UnicodeNoStr(object):
    def __unicode__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'

class StrNoUnicode(object):
    def __str__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'.encode('utf8')

class StrReturnsUnicode(object):
    def __str__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'

class UnicodeReturnsStr(object):
    def __unicode__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'.encode('utf8')

class UnicodeStrCrossed(object):
    def __unicode__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'.encode('utf8')

    def __str__(self):
        return 'El veloz murciélago saltó sobre el perro perezoso.'

class ReprUnicode(object):
    def __repr__(self):
        return 'ReprUnicode(El veloz murciélago saltó sobre el perro perezoso.)'

class TestConverters(unittest.TestCase, base_classes.UnicodeTestData):
    def test_to_unicode(self):
        '''Test to_unicode when the user gives good values'''
        tools.eq_(converters.to_unicode(self.u_japanese, encoding='latin1'), self.u_japanese)

        tools.eq_(converters.to_unicode(self.utf8_spanish), self.u_spanish)
        tools.eq_(converters.to_unicode(self.utf8_japanese), self.u_japanese)

        tools.eq_(converters.to_unicode(self.latin1_spanish, encoding='latin1'), self.u_spanish)
        tools.eq_(converters.to_unicode(self.euc_jp_japanese, encoding='euc_jp'), self.u_japanese)

        tools.assert_raises(TypeError, converters.to_unicode, *[5], **{'nonstring': 'foo'})

    def test_to_unicode_errors(self):
        tools.eq_(converters.to_unicode(self.latin1_spanish), self.u_mangled_spanish_latin1_as_utf8)
        tools.eq_(converters.to_unicode(self.latin1_spanish, errors='ignore'), self.u_spanish_ignore)
        tools.assert_raises(UnicodeDecodeError, converters.to_unicode,
                *[self.latin1_spanish], **{'errors': 'strict'})

    def test_to_unicode_nonstring(self):
        tools.eq_(converters.to_unicode(5), '5')
        tools.eq_(converters.to_unicode(5, nonstring='empty'), '')
        tools.eq_(converters.to_unicode(5, nonstring='passthru'), 5)
        tools.eq_(converters.to_unicode(5, nonstring='simplerepr'), '5')
        tools.eq_(converters.to_unicode(5, nonstring='repr'), '5')
        tools.assert_raises(TypeError, converters.to_unicode, *[5], **{'nonstring': 'strict'})

        obj_repr = converters.to_unicode(object, nonstring='simplerepr')
        tools.eq_(obj_repr, "<class 'object'>")
        tools.assert_true(isinstance(obj_repr, str))

    def test_to_unicode_nonstring_with_objects_that_have__unicode__and__str__(self):
        '''Test that to_unicode handles objects that have  __unicode__ and  __str__ methods'''
        if sys.version_info < (3, 0):
            # None of these apply on python3 because python3 does not use __unicode__
            # and it enforces __str__ returning str
            tools.eq_(converters.to_unicode(UnicodeNoStr(), nonstring='simplerepr'), self.u_spanish)
            tools.eq_(converters.to_unicode(StrNoUnicode(), nonstring='simplerepr'), self.u_spanish)
            tools.eq_(converters.to_unicode(UnicodeReturnsStr(), nonstring='simplerepr'), self.u_spanish)

        tools.eq_(converters.to_unicode(StrReturnsUnicode(), nonstring='simplerepr'), self.u_spanish)
        tools.eq_(converters.to_unicode(UnicodeStrCrossed(), nonstring='simplerepr'), self.u_spanish)

    def test_to_bytes(self):
        '''Test to_bytes when the user gives good values'''
        tools.eq_(converters.to_bytes(self.utf8_japanese, encoding='latin1'), self.utf8_japanese)

        tools.eq_(converters.to_bytes(self.u_spanish), self.utf8_spanish)
        tools.eq_(converters.to_bytes(self.u_japanese), self.utf8_japanese)

        tools.eq_(converters.to_bytes(self.u_spanish, encoding='latin1'), self.latin1_spanish)
        tools.eq_(converters.to_bytes(self.u_japanese, encoding='euc_jp'), self.euc_jp_japanese)

    def test_to_bytes_errors(self):
        tools.eq_(converters.to_bytes(self.u_mixed, encoding='latin1'),
                self.latin1_mixed_replace)
        tools.eq_(converters.to_bytes(self.u_mixed, encoding='latin',
            errors='ignore'), self.latin1_mixed_ignore)
        tools.assert_raises(UnicodeEncodeError, converters.to_bytes,
            *[self.u_mixed], **{'errors': 'strict', 'encoding': 'latin1'})

    def _check_repr_bytes(self, repr_string, obj_name):
        tools.assert_true(isinstance(repr_string, bytes))
        match = self.repr_re.match(repr_string)
        tools.assert_not_equal(match, None)
        tools.eq_(match.groups()[0], obj_name)

    def test_to_bytes_nonstring(self):
        tools.eq_(converters.to_bytes(5), b'5')
        tools.eq_(converters.to_bytes(5, nonstring='empty'), b'')
        tools.eq_(converters.to_bytes(5, nonstring='passthru'), 5)
        tools.eq_(converters.to_bytes(5, nonstring='simplerepr'), b'5')
        tools.eq_(converters.to_bytes(5, nonstring='repr'), b'5')

        # Raise a TypeError if the msg is nonstring and we're set to strict
        tools.assert_raises(TypeError, converters.to_bytes, *[5], **{'nonstring': 'strict'})
        # Raise a TypeError if given an invalid nonstring arg
        tools.assert_raises(TypeError, converters.to_bytes, *[5], **{'nonstring': 'INVALID'})

        obj_repr = converters.to_bytes(object, nonstring='simplerepr')
        tools.eq_(obj_repr, b"<class 'object'>")
        tools.assert_true(isinstance(obj_repr, bytes))

    def test_to_bytes_nonstring_with_objects_that_have__unicode__and__str__(self):
        if sys.version_info < (3, 0):
            # This object's _str__ returns a utf8 encoded object
            tools.eq_(converters.to_bytes(StrNoUnicode(), nonstring='simplerepr'), self.utf8_spanish)
        # No __str__ method so this returns repr
        string = converters.to_bytes(UnicodeNoStr(), nonstring='simplerepr')
        self._check_repr_bytes(string, b'UnicodeNoStr')

        # This object's __str__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(StrReturnsUnicode(), nonstring='simplerepr'), self.utf8_spanish)
        # Unless we explicitly ask for something different
        tools.eq_(converters.to_bytes(StrReturnsUnicode(),
            nonstring='simplerepr', encoding='latin1'), self.latin1_spanish)

        # This object has no __str__ so it returns repr
        string = converters.to_bytes(UnicodeReturnsStr(), nonstring='simplerepr')
        self._check_repr_bytes(string, b'UnicodeReturnsStr')

        # This object's __str__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(UnicodeStrCrossed(), nonstring='simplerepr'), self.utf8_spanish)

        # This object's __repr__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(ReprUnicode(), nonstring='simplerepr'),
                'ReprUnicode(El veloz murciélago saltó sobre el perro perezoso.)'.encode('utf8'))
        tools.eq_(converters.to_bytes(ReprUnicode(), nonstring='repr'),
                'ReprUnicode(El veloz murciélago saltó sobre el perro perezoso.)'.encode('utf8'))

    def test_unicode_to_xml(self):
        tools.eq_(converters.unicode_to_xml(None), b'')
        tools.assert_raises(XmlEncodeError, converters.unicode_to_xml, *[b'byte string'])
        tools.assert_raises(ValueError, converters.unicode_to_xml, *['string'], **{'control_chars': 'foo'})
        tools.assert_raises(XmlEncodeError, converters.unicode_to_xml,
                *['string\u0002'], **{'control_chars': 'strict'})
        tools.eq_(converters.unicode_to_xml(self.u_entity), self.utf8_entity_escape)
        tools.eq_(converters.unicode_to_xml(self.u_entity, attrib=True), self.utf8_attrib_escape)
        tools.eq_(converters.unicode_to_xml(self.u_entity, encoding='ascii'), self.ascii_entity_escape)
        tools.eq_(converters.unicode_to_xml(self.u_entity, encoding='ascii', attrib=True), self.ascii_attrib_escape)

    def test_xml_to_unicode(self):
        tools.eq_(converters.xml_to_unicode(self.utf8_entity_escape, 'utf8', 'replace'), self.u_entity)
        tools.eq_(converters.xml_to_unicode(self.utf8_attrib_escape, 'utf8', 'replace'), self.u_entity)
        tools.eq_(converters.xml_to_unicode(self.ascii_entity_escape, 'ascii', 'replace'), self.u_entity)
        tools.eq_(converters.xml_to_unicode(self.ascii_attrib_escape, 'ascii', 'replace'), self.u_entity)

    def test_xml_to_byte_string(self):
        tools.eq_(converters.xml_to_byte_string(self.utf8_entity_escape, 'utf8', 'replace'), self.u_entity.encode('utf8'))
        tools.eq_(converters.xml_to_byte_string(self.utf8_attrib_escape, 'utf8', 'replace'), self.u_entity.encode('utf8'))
        tools.eq_(converters.xml_to_byte_string(self.ascii_entity_escape, 'ascii', 'replace'), self.u_entity.encode('utf8'))
        tools.eq_(converters.xml_to_byte_string(self.ascii_attrib_escape, 'ascii', 'replace'), self.u_entity.encode('utf8'))

        tools.eq_(converters.xml_to_byte_string(self.utf8_attrib_escape,
            output_encoding='euc_jp', errors='replace'),
            self.u_entity.encode('euc_jp', 'replace'))
        tools.eq_(converters.xml_to_byte_string(self.utf8_attrib_escape,
            output_encoding='latin1', errors='replace'),
            self.u_entity.encode('latin1', 'replace'))
        tools.eq_(converters.xml_to_byte_string(self.ascii_attrib_escape,
            output_encoding='euc_jp', errors='replace'),
            self.u_entity.encode('euc_jp', 'replace'))
        tools.eq_(converters.xml_to_byte_string(self.ascii_attrib_escape,
            output_encoding='latin1', errors='replace'),
            self.u_entity.encode('latin1', 'replace'))

    def test_byte_string_to_xml(self):
        tools.assert_raises(XmlEncodeError, converters.byte_string_to_xml, *['test'])
        tools.eq_(converters.byte_string_to_xml(self.utf8_entity), self.utf8_entity_escape)
        tools.eq_(converters.byte_string_to_xml(self.utf8_entity, attrib=True), self.utf8_attrib_escape)

    def test_bytes_to_xml(self):
        tools.eq_(converters.bytes_to_xml(self.b_byte_chars), self.b_byte_encoded)

    def test_xml_to_bytes(self):
        tools.eq_(converters.xml_to_bytes(self.b_byte_encoded), self.b_byte_chars)

    def test_guess_encoding_to_xml(self):
        tools.eq_(converters.guess_encoding_to_xml(self.u_entity), self.utf8_entity_escape)
        tools.eq_(converters.guess_encoding_to_xml(self.utf8_spanish), self.utf8_spanish)
        tools.eq_(converters.guess_encoding_to_xml(self.latin1_spanish), self.utf8_spanish)
        tools.eq_(converters.guess_encoding_to_xml(self.utf8_japanese), self.utf8_japanese)

    def test_guess_encoding_to_xml_euc_japanese(self):
        if chardet:
            tools.eq_(converters.guess_encoding_to_xml(self.euc_jp_japanese),
                    self.utf8_japanese)
        else:
            raise SkipTest('chardet not installed, euc_japanese won\'t be detected')

    def test_guess_encoding_to_xml_euc_japanese_mangled(self):
        if chardet:
            raise SkipTest('chardet installed, euc_japanese won\'t be mangled')
        else:
            tools.eq_(converters.guess_encoding_to_xml(self.euc_jp_japanese),
                    self.utf8_mangled_euc_jp_as_latin1)

class TestGetWriter(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.io = io.BytesIO()

    def test_utf8_writer(self):
        writer = converters.getwriter('utf-8')
        io = writer(self.io)
        io.write(self.u_japanese + '\n')
        io.seek(0)
        result = io.read().strip()
        tools.eq_(result, self.utf8_japanese)

        io.seek(0)
        io.truncate(0)
        io.write(self.euc_jp_japanese + b'\n')
        io.seek(0)
        result = io.read().strip()
        tools.eq_(result, self.euc_jp_japanese)

        io.seek(0)
        io.truncate(0)
        io.write(self.utf8_japanese + b'\n')
        io.seek(0)
        result = io.read().strip()
        tools.eq_(result, self.utf8_japanese)

    def test_error_handlers(self):
        '''Test setting alternate error handlers'''
        writer = converters.getwriter('latin1')
        io = writer(self.io, errors='strict')
        tools.assert_raises(UnicodeEncodeError, io.write, self.u_japanese)


class TestExceptionConverters(unittest.TestCase, base_classes.UnicodeTestData):
    def setUp(self):
        self.exceptions = {}
        tests = {'u_jpn': self.u_japanese,
                'u_spanish': self.u_spanish,
                'utf8_jpn': self.utf8_japanese,
                'utf8_spanish': self.utf8_spanish,
                'euc_jpn': self.euc_jp_japanese,
                'latin1_spanish': self.latin1_spanish}
        for test in tests.items():
            try:
                raise Exception(test[1])
            except Exception as xxx_todo_changeme:
                self.exceptions[test[0]] = xxx_todo_changeme
                pass

    def test_exception_to_unicode_with_unicode(self):
        tools.eq_(converters.exception_to_unicode(self.exceptions['u_jpn']), self.u_japanese)
        tools.eq_(converters.exception_to_unicode(self.exceptions['u_spanish']), self.u_spanish)

    def test_exception_to_unicode_with_bytes(self):
        tools.eq_(converters.exception_to_unicode(self.exceptions['utf8_jpn']), self.u_japanese)
        tools.eq_(converters.exception_to_unicode(self.exceptions['utf8_spanish']), self.u_spanish)
        # Mangled latin1/utf8 conversion but no tracebacks
        tools.eq_(converters.exception_to_unicode(self.exceptions['latin1_spanish']), self.u_mangled_spanish_latin1_as_utf8)
        # Mangled euc_jp/utf8 conversion but no tracebacks
        tools.eq_(converters.exception_to_unicode(self.exceptions['euc_jpn']), self.u_mangled_euc_jp_as_utf8)

    def test_exception_to_unicode_custom(self):
        # If given custom functions, then we should not mangle
        c = [lambda e: converters.to_unicode(e.args[0], encoding='euc_jp'),
                lambda e: converters.to_unicode(e, encoding='euc_jp')]
        tools.eq_(converters.exception_to_unicode(self.exceptions['euc_jpn'],
            converters=c), self.u_japanese)
        c.extend(converters.EXCEPTION_CONVERTERS)
        tools.eq_(converters.exception_to_unicode(self.exceptions['euc_jpn'],
            converters=c), self.u_japanese)

        c = [lambda e: converters.to_unicode(e.args[0], encoding='latin1'),
                lambda e: converters.to_unicode(e, encoding='latin1')]
        tools.eq_(converters.exception_to_unicode(self.exceptions['latin1_spanish'],
            converters=c),  self.u_spanish)
        c.extend(converters.EXCEPTION_CONVERTERS)
        tools.eq_(converters.exception_to_unicode(self.exceptions['latin1_spanish'],
            converters=c),  self.u_spanish)

    def test_exception_to_bytes_with_unicode(self):
        tools.eq_(converters.exception_to_bytes(self.exceptions['u_jpn']), self.utf8_japanese)
        tools.eq_(converters.exception_to_bytes(self.exceptions['u_spanish']), self.utf8_spanish)

    def test_exception_to_bytes_with_bytes(self):
        tools.eq_(converters.exception_to_bytes(self.exceptions['utf8_jpn']), self.utf8_japanese)
        tools.eq_(converters.exception_to_bytes(self.exceptions['utf8_spanish']), self.utf8_spanish)
        tools.eq_(converters.exception_to_bytes(self.exceptions['latin1_spanish']), self.latin1_spanish)
        tools.eq_(converters.exception_to_bytes(self.exceptions['euc_jpn']), self.euc_jp_japanese)

    def test_exception_to_bytes_custom(self):
        # If given custom functions, then we should not mangle
        c = [lambda e: converters.to_bytes(e.args[0], encoding='euc_jp'),
                lambda e: converters.to_bytes(e, encoding='euc_jp')]
        tools.eq_(converters.exception_to_bytes(self.exceptions['euc_jpn'],
            converters=c), self.euc_jp_japanese)
        c.extend(converters.EXCEPTION_CONVERTERS)
        tools.eq_(converters.exception_to_bytes(self.exceptions['euc_jpn'],
            converters=c), self.euc_jp_japanese)

        c = [lambda e: converters.to_bytes(e.args[0], encoding='latin1'),
                lambda e: converters.to_bytes(e, encoding='latin1')]
        tools.eq_(converters.exception_to_bytes(self.exceptions['latin1_spanish'],
            converters=c),  self.latin1_spanish)
        c.extend(converters.EXCEPTION_CONVERTERS)
        tools.eq_(converters.exception_to_bytes(self.exceptions['latin1_spanish'],
            converters=c),  self.latin1_spanish)


class TestDeprecatedConverters(TestConverters):
    def setUp(self):
        warnings.simplefilter('ignore', DeprecationWarning)

    def tearDown(self):
        warnings.simplefilter('default', DeprecationWarning)

    def test_to_xml(self):
        tools.eq_(converters.to_xml(self.u_entity), self.utf8_entity_escape)
        tools.eq_(converters.to_xml(self.utf8_spanish), self.utf8_spanish)
        tools.eq_(converters.to_xml(self.latin1_spanish), self.utf8_spanish)
        tools.eq_(converters.to_xml(self.utf8_japanese), self.utf8_japanese)

    def test_to_utf8(self):
        tools.eq_(converters.to_utf8(self.u_japanese), self.utf8_japanese)
        tools.eq_(converters.to_utf8(self.utf8_spanish), self.utf8_spanish)

    def test_to_str(self):
        tools.eq_(converters.to_str(self.u_japanese), self.utf8_japanese)
        tools.eq_(converters.to_str(self.utf8_spanish), self.utf8_spanish)
        tools.eq_(converters.to_str(object), b"<class 'object'>")

    def test_non_string(self):
        '''Test deprecated non_string parameter'''
        # unicode
        tools.assert_raises(TypeError, converters.to_unicode, *[5], **{'non_string': 'foo'})
        tools.eq_(converters.to_unicode(5, non_string='empty'), '')
        tools.eq_(converters.to_unicode(5, non_string='passthru'), 5)
        tools.eq_(converters.to_unicode(5, non_string='simplerepr'), '5')
        tools.eq_(converters.to_unicode(5, non_string='repr'), '5')
        tools.assert_raises(TypeError, converters.to_unicode, *[5], **{'non_string': 'strict'})

        tools.eq_(converters.to_unicode(StrReturnsUnicode(), non_string='simplerepr'), self.u_spanish)
        tools.eq_(converters.to_unicode(UnicodeStrCrossed(), non_string='simplerepr'), self.u_spanish)
        if sys.version_info < (3, 0):
            # Not applicable on python3
            tools.eq_(converters.to_unicode(UnicodeNoStr(), non_string='simplerepr'), self.u_spanish)
            tools.eq_(converters.to_unicode(StrNoUnicode(), non_string='simplerepr'), self.u_spanish)
            tools.eq_(converters.to_unicode(UnicodeReturnsStr(), non_string='simplerepr'), self.u_spanish)

        obj_repr = converters.to_unicode(object, non_string='simplerepr')
        tools.eq_(obj_repr, "<class 'object'>")
        tools.assert_true(isinstance(obj_repr, str))

        # Bytes
        tools.eq_(converters.to_bytes(5), b'5')
        tools.eq_(converters.to_bytes(5, non_string='empty'), b'')
        tools.eq_(converters.to_bytes(5, non_string='passthru'), 5)
        tools.eq_(converters.to_bytes(5, non_string='simplerepr'), b'5')
        tools.eq_(converters.to_bytes(5, non_string='repr'), b'5')

        # Raise a TypeError if the msg is non_string and we're set to strict
        tools.assert_raises(TypeError, converters.to_bytes, *[5], **{'non_string': 'strict'})
        # Raise a TypeError if given an invalid non_string arg
        tools.assert_raises(TypeError, converters.to_bytes, *[5], **{'non_string': 'INVALID'})

        # No __str__ method so this returns repr
        string = converters.to_bytes(UnicodeNoStr(), non_string='simplerepr')
        self._check_repr_bytes(string, b'UnicodeNoStr')

        if sys.version_info < (3, 0):
            # Not applicable on python3
            # This object's _str__ returns a utf8 encoded object
            tools.eq_(converters.to_bytes(StrNoUnicode(), non_string='simplerepr'), self.utf8_spanish)

        # This object's __str__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(StrReturnsUnicode(), non_string='simplerepr'), self.utf8_spanish)
        # Unless we explicitly ask for something different
        tools.eq_(converters.to_bytes(StrReturnsUnicode(),
            non_string='simplerepr', encoding='latin1'), self.latin1_spanish)

        # This object has no __str__ so it returns repr
        string = converters.to_bytes(UnicodeReturnsStr(), non_string='simplerepr')
        self._check_repr_bytes(string, b'UnicodeReturnsStr')

        # This object's __str__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(UnicodeStrCrossed(), non_string='simplerepr'), self.utf8_spanish)

        # This object's __repr__ returns unicode which to_bytes converts to utf8
        tools.eq_(converters.to_bytes(ReprUnicode(), non_string='simplerepr'),
                'ReprUnicode(El veloz murciélago saltó sobre el perro perezoso.)'.encode('utf8'))
        tools.eq_(converters.to_bytes(ReprUnicode(), non_string='repr'),
                'ReprUnicode(El veloz murciélago saltó sobre el perro perezoso.)'.encode('utf8'))

        obj_repr = converters.to_bytes(object, non_string='simplerepr')
        tools.eq_(obj_repr, b"<class 'object'>")
        tools.assert_true(isinstance(obj_repr, bytes))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2010 Red Hat, Inc
#
# This file is part of kitchen
#
# kitchen is free software; you can redistribute it and/or modify it under the
# terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
#
# kitchen is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
# more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>

'''
In python-2.4, a builtin :class:`set` type was added to python.  This module
provides a function to emulate that on python-2.3 by using the :mod:`sets`
module.

:func:`set`
    Create a set.  If running on python 2.4+ this is the :class:`set`
    constructor.  If using python-2.3, it's :class:`sets.Set`.

:func:`frozenset`
    Create a frozenset.  If running on python2.4+ this is the
    :class:`frozenset` constructor.  If using python-2.3, it's
    :class:`sets.ImmutableSet`.

.. versionchanged:: 0.2.0 API: kitchen.pycompat24 1.0.0
    Added set and frozenset
'''
import __builtin__

# Setup set and frozenset on this module
# :W0622,C0103: The purpose of this module is to define set and frozenset if
#   they aren't in builtins already so we disregard these pylint warnings
#pylint:disable-msg=W0622,C0103
if not hasattr(__builtin__, 'set'):
    import sets
    set = sets.Set
else:
    set = set

if not hasattr(__builtin__, 'frozenset'):
    import sets
    frozenset = sets.ImmutableSet
else:
    frozenset = frozenset
#pylint:enable-msg=W0622,C0103

def add_builtin_set():
    '''If there's no set builtin, us the :mod:`sets` module to make one

    This function makes sure that a :class:`set` and :class:`frozenset` type
    are available in the :mod:`__builtin__` namespace.  Since the function
    checks whether :class:`set` and :class:`frozenset` are already present in
    the :mod:`__builtin__` namespace and refuses to overwrite those if found,
    it's safe to call this in multiple places and in scripts run under
    python-2.4+, where a more efficient set implementation is already present
    in the :mod:`__builtin__` namespace.

    However, since this function modifies :mod:`__builtin__` there's no need
    to call it more than once  so you likely want to do something like this
    when your program loads::

        myprogram/__init__.py:

        from kitchen.pycompat24 import sets
        builtinset.add_builtin_set()

    You can then use :func:`set` and :func:`frozenset` anywhere in your code::

        myprogram/compute.py:

        def math_students(algebra_student_list, geometry_student_list):
            return set(algebra_student_list) union set(geometry_student_list)
    '''
    if not hasattr(__builtin__, 'set'):
        __builtin__.set = set

    if not hasattr(__builtin__, 'frozenset'):
        __builtin__.frozenset = frozenset

__all__ = ('add_builtin_set', 'set', 'frozenset')



# -*- coding: utf-8 -*-
#
# Copyright (c) 2012 Red Hat, Inc
#
# kitchen is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# kitchen is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with kitchen; if not, see <http://www.gnu.org/licenses/>
#
# Authors:
#   Toshio Kuratomi <toshio@fedoraproject.org>
'''
----------
StrictDict
----------

:class:`kitchen.collections.StrictDict` provides a dictionary that treats
:class:`bytes` and :class:`str` as distinct key values.
'''

# Pylint disabled messages:
# :C0111: We're implementing the dict interface so just reference the dict
#   documentation rather than having our own docstrings

import warnings

warnings.warn('In python3, kitchen.collections.strictdict is deprecated.'
        '  If your code doesn\'t have to remain compatible with python2 use'
        ' python3\'s native dict or defaultdict types instead',
        PendingDeprecationWarning, stacklevel=2)

try:
    # :E0611: Pylint false positive.  We try to import from the stdlib but we
    #   have a fallback so this is okay.
    #pylint:disable-msg=E0611
    from collections import defaultdict
except ImportError:
    from kitchen.pycompat25.collections import defaultdict

# in python3, ordinary dictionaries keep bytes and strings separate
StrictDict = defaultdict

__all__ = ('StrictDict',)
